#!/bin/bash

. globals.sh
. parse-iptl.sh
. write-svg.sh
. rule.sh
. box.sh

if [ "$1" == "--list-plugins" ]
then
    echo "Installed plugins:"
    echo "=================="

    tmpfile=`mktemp`

    ls -1 $PLUGINS_DATADIR > $tmpfile

    while read line 
    do
      echo "$line: `head -n 1 $PLUGINS_DATADIR/$line`"
    done < $tmpfile

    rm -f $tmpfile

    exit
fi

if [ $# -lt 2 ] || [ "$1" == "--help" ]
then
    echo ""
    echo "  Syntax: $0 [filename.iptl] --pluginname"
    echo "          $0 --help"
    echo "          $0 --list-plugins"
    echo ""
    echo "  Where \"filename.iptl\" is the result of \"iptables -L\""
    echo "  And pluginname an installed plugin."
    echo ""
    echo " Example: $0 file.iptables --svg"
    echo ""
    exit
fi

function load_plugin () {

    plugin=$1

    tmpfile=`mktemp`
    ls -1 $PLUGINS_DATADIR > $tmpfile
    
    while read line 
      do
      if [ "--$line" == "$plugin" ]
      then
	  . $PLUGINS_DATADIR/$line
      fi
    done < $tmpfile
    
    rm -f $tmpfile

}

load_plugin $2

plugin_start

parse_file $1
plugin_run

plugin_end

rule_group "INPUT" source
