#!/bin/bash
#
# This is bash++ !!
# It handles object feeding and properties to return 
#

#
# datastructure:
# OBJCHAINS_LIST 
#     |
#     +--> "chain_name" @chaindata (1)
#                         |
#                         +--> @rules @graphic (2)
#                                |       |
#                                |       +--> "graphic_type" @coords
#                                |                              |
#                                |                              +--> x1, y1, x2/width, y2/height 
#                                |
#                                +--> target, proto, opt, source, destination
# 
# (1) object_chain_add, object_chain_get_first, object_chain_get_all
# (2) object_rule_graphic_get
#

#
# Usage:
#
# chaindata=()
# object_chain_add "INPUT" "`echo ${chaindata[@]}`"
# chainarray=( `object_chain_get "INPUT"` )
# echo ${chainarray[3]}
#

objchains_list=()

#
# Add a chain
# $1: STRING name of the chain
# $2: ARRAY  chain data 
#
function object_chain_add () {
    chain_name=$1
    data=$2

    objchainslen=${#objchains_list[@]}
    objchains_list[objchainslen]=$chain_name

    objchainslen=${#objchains_list[@]}
    objchains_list[objchainslen]=${data[@]}
}

#
# Get a specified chain in parameter
#
function object_chain_get_first () {
    chain_name=$1

    i=0

    objectlistlen=${#objchains_list[@]}

    while [ $i -lt $objectlistlen ]
    do

      object=${objchains_list[$i]}

      if [ "$object" == "$chain_name" ]
      then
	  let "i += 1"
	  echo "${objchains_list[$i]}"
	  return
      fi
      
      let "i += 2"   # "chain_name1" @data "chain_name2" @data "chain_name3" @data ...
                     #       0                   2                   4
    done
}

#
# Extract all rules from a chain
#
function object_chain_get_all () {
    chain_name=$1

    i=0

    objectlistlen=${#objchains_list[@]}

    while [ $i -lt $objectlistlen ]
    do

      object=${objchains_list[$i]}

      if [ "$object" == "$chain_name" ]
      then
	  let "i += 1"
	  echo "${objchains_list[$i]}"
	  let "i -= 1"
      fi
      
      let "i += 2"   # "chain_name1" @data "chain_name2" @data "chain_name3" @data ...
                     #       0                   2                   4
    done
}

#
# Count the number of rules in a given chain
#
function object_chain_count () {
    chain_name=$1
    
    i=0
    count=0

    objectlistlen=${#objchains_list[@]}

    while [ $i -lt $objectlistlen ]
    do

      object=${objchains_list[$i]}

      if [ "$object" == "$chain_name" ]
      then
	  let "count += 1"
      fi
      
      let "i += 2"   # "chain_name1" @data "chain_name2" @data "chain_name3" @data ...
                     #       0                   2                   4
    done
    
    echo $count
}

#
# Count the number of rules in a given chain
#
function object_chain_get_rule_number () {
    chain_name=$1
    number=$2

    i=0
    count=0

    objectlistlen=${#objchains_list[@]}

    while [ $i -lt $objectlistlen ]
    do

      object=${objchains_list[$i]}

      if [ "$object" == "$chain_name" ]
      then
	  let "count += 1"
	  if [ $count -eq $number ]
	  then
	      let "i += 1"
	      echo "${objchains_list[$i]}"
	      return
	  fi
      fi
      
      let "i += 2"   # "chain_name1" @data "chain_name2" @data "chain_name3" @data ...
                     #       0                   2                   4
    done
}


#
# Feed a rule to a chain and get its graphical properties
# Usage: graphicarray=( `object_rule_graphic_get "OUTPUT" "target3 proto3 opt3 source3 destination3"` )
#
function object_rule_graphic_get () {
    chain_name=$1
    rule=$2

    i=0

    objectlistlen=${#objchains_list[@]}

    while [ $i -lt $objectlistlen ]
    do

      object=${objchains_list[$i]}

      if [ "$object" == "$chain_name" ]
      then
	  let "i += 1"

	  ruleobject=${objchains_list[$i]}
	  rulerule=`echo $ruleobject | cut -f 1 -d \|`
	  rulegraphic=`echo $ruleobject | cut -f 2 -d \|`
	  if [ "$rulerule" == "$rule " ] # Extra space required
	  then
	      echo $rulegraphic
	  fi

	  let "i -= 1"
      fi
      
      let "i += 2"   # "chain_name1" @data "chain_name2" @data "chain_name3" @data ...
                     #       0                   2                   4
    done
}

#chaindata=("target proto opt source destination | line 12 34 43 12")
#object_chain_add "INPUT" "`echo ${chaindata[@]}`"
#chaindata=("target2 proto2 opt2 source2 destination2 | line 122 342 432 122")
#object_chain_add "INPUT" "`echo ${chaindata[@]}`"
#chaindata=("target3 proto3 opt3 source3 destination3 | line 133 343 433 133")
#object_chain_add "OUTPUT" "`echo ${chaindata[@]}`"

#chainarray=( `object_chain_get_all "OUTPUT"` )

#chainarraylen=${#chainarray[@]}
#echo $chainarraylen
# i=0

# while [ $i -lt $chainarraylen ]
# do
#   echo ${chainarray[$i]}
#   let "i += 1"
# done

#graphicarray=( `object_rule_graphic_get "OUTPUT" "target3 proto3 opt3 source3 destination3"` )
#echo "MA=${graphicarray[@]}"
