#!/bin/bash

#
# parse-iptl.sh
# 
# [ iptables -L ] --> DATASTRUCTURE
# The DATASTRUCTURE is then given to the plugin framework (see fw2foo.sh) 
#

. policy.sh
. objects.sh

chains_set_line_pos () {
#
# We make an global array storing the line position in the iptables file 
# given so that we can then parse the file just for a given chain
# The array looks like: ( INPUT 1 FORWARD 4 OUTPUT 7 )
#

    chain=$1
    line=$2

    CHAINSLOCATE_LASTPOS=${#FW2SVG_CHAINSLOCATE[@]}
    FW2SVG_CHAINSLOCATE[CHAINSLOCATE_LASTPOS]=$chain
    let "CHAINSLOCATE_LASTPOS += 1"
    FW2SVG_CHAINSLOCATE[CHAINSLOCATE_LASTPOS]=$line

}

parse_policies () {
#
# We get the chain and its policy
# * Adds the policy into the SVG file
# * Make an array with chain line position 
#

    filename=$1
    i=0

    while read line
    do
      let "i += 1"
      CHAIN_KEYWORD=`echo $line |awk '{print $1}'`

      if [ "$CHAIN_KEYWORD" == "Chain" ]
      then
	  PLEN=`echo $line |awk '{print $4}'` # We should drop the extra ) character
	  PLEN=`echo ${#PLEN}`
	  let "PLEN -= 1"
	  POLICY=`echo $line |awk '{print $4}' |cut -b -$PLEN`
	  CHAIN=`echo $line |awk '{print $2}'`

	  policy_add $CHAIN $POLICY

	  chains_set_line_pos $CHAIN $i
      fi
    
    done <$filename

}

parse_chain () {
    filename=$1
    chain_to_parse=$2

    i=0
    j=0
    nb_chain_rule=0

    box_width=0
    box_height=0

#
# First Step: Grab where the CHAINS starts and where it ends
#
    CHAINSLOCATE_LASTPOS=${#FW2SVG_CHAINSLOCATE[@]}
    while [ $j -lt $CHAINSLOCATE_LASTPOS ]
    do
      if [ $chain_to_parse == ${FW2SVG_CHAINSLOCATE[$j]} ]
      then
	  let "j += 1" # We jump to get the next element which returns the start position
	  chain_pos_start=${FW2SVG_CHAINSLOCATE[$j]}
	  let "chain_pos_start += 1" # To drop the title "target prot opt source destination"
	  let "j += 2" # We jump to next positions to get the end position
	  chain_pos_end=${FW2SVG_CHAINSLOCATE[$j]}
	  let "chain_pos_end -= 1" # We don't want the line concerning the last chain
	                           # If we are in the last chain then -1 is returned instead
      fi

      let "j += 1"
    done


    CURRENT_Y=$FW2SVG_HEIGHT   # Useful to get average height

    while read line
    do
      let "i += 1"

      if [ $i -gt $chain_pos_start ] && [ $i -lt $chain_pos_end ]
      then
	  # $1 = target; $2 = proto; $3 = opt; $4 = source; $5 = destination
	  target=`echo $line | awk '{print $1}'`
	  proto=`echo $line | awk '{print $2}'`
	  opt=`echo $line | awk '{print $3}'`
	  source=`echo $line | awk '{print $4}'`
	  destination=`echo $line | awk '{print $5}'`

      #
      # Start: Object Management
      # 
	  #
	  # We must calculate the INPUT source width
	  # FIXME: This has to be done in a more generic manner
	  #
	  sourcelen=${#source}
	  let "current_width = $sourcelen * 12"
# 	  if [ $WIDTH -gt $FW2SVG_WIDTH ]
# 	  then
# 	      FW2SVG_WIDTH=$WIDTH
# 	  fi
	  if [ $current_width -gt $box_width ]
	  then
	      box_width=$current_width
	  fi

	  let "box_height += $MARGIN_BOX_TOP"

	  object_chain_add "$chain_to_parse" "$target $proto $opt $source $destination | rect $MARGIN_BOX_LEFT $box_height $current_width $FW2SVG_BOX_HEIGHT"
	  let "box_height += $FW2SVG_BOX_HEIGHT"

      #
      # End: Object Management
      #

# 	  write_svg_box $CURRENT_Y $chain_to_parse $target $proto $opt $source $destination

	  let "nb_chain_rule += 1"
      fi
    
    done <$filename

    box_new $2 "0 0 $box_width $box_height"

}

parse_file () {

    parse_policies $1
#    write_svg_chain "INPUT"
    parse_chain $1 "INPUT"

#    write_svg_machine "INPUT"

}
